/**
 * @file bipartitegraph.cpp
 * @author Léo Perrin
 * @date 2011/11/23
 */

#include <stdlib.h>
#include <vector>
#include <fstream>
#include <iostream>


#include "vertex.h"
#include "bipartitegraph.h"

/**
 * @def MAX_SIZE
 * The maximal size of a line in a file detailing a graph's structure.
 */
#define MAX_SIZE 256



BipartiteGraph::BipartiteGraph(std::istream * input)
{
	unsigned int m,n;
	(*input) >> m;
	(*input) >> n;
	char buff[MAX_SIZE];
	(*input).getline(buff,MAX_SIZE); // A line is jump, so it is
			// necessary to call it "emptily".
	// building left vertices from the input
	while ((*input).getline(buff,MAX_SIZE))
	{
		std::string line (buff);
		leftVertices.push_back(Vertex::line2vertex(line));
	}
	// Building right vertices from the left ones
	for (unsigned int i=0; i<n; i++)
	{
		Vertex right(i);
		rightVertices.push_back(right);
	}
	// Adding their neighbours to the right vertices
	for (unsigned int i=0; i<leftVertices.size(); i++)
		for (unsigned int j=0; j<leftVertices[i].order();j++)
			rightVertices[ leftVertices[i].getNeighbours()[j] ] \
				.addNeighbour(leftVertices[i].getIndex());
}


unsigned int BipartiteGraph::getNleftVertices()
{
	return leftVertices.size();
}


unsigned int BipartiteGraph::getNrightVertices()
{
	return rightVertices.size();
}


Vertex BipartiteGraph::leftVertex(unsigned int ind)
{
	if (ind >= leftVertices.size())
	{
		std::cout << "error 1100: no such left vertex, index is too big!"
		     << std::endl;
		exit(1100);
	}
	else
		return leftVertices[ind];
}


Vertex BipartiteGraph::rightVertex(unsigned int ind)
{
	if (ind >= rightVertices.size())
	{
		std::cout << "error 1101: no such right vertex, index is too big!"
		     << std::endl;
		exit(1101);
	}
	else
		return rightVertices[ind];
}
