#include <stdlib.h>
#include <vector>
#include <iostream>
#include <fstream>
#include <string>

#include "../libcnf/libcnf.h"

using namespace cnf;


/**
 * @brief Takes an input stream to an assignment dimacs file and returns
 * the full sudoku grid corresponding to this assignment.
 */
std::vector< std::vector<int> > assignment2Grid(std::istream * input)
{
	std::vector< std::vector<int> > grid;
	VariableSet vars(9,9,9);
	vars.parseDimacs(input);
	if (vars.getCard() < 729)
	{
		std::cout << "error 400: Not enough variables in the file!" << std::endl;
		exit(400);
	}
	else
	{
		bool found;
		for (unsigned int i=0; i<9; i++)
		{
			std::vector<int> line;
			std::cout << std::endl;
			for (unsigned int j=0; j<9; j++)
			{
				found = false;
				for (unsigned int k=0; k<9; k++)
					if (vars.getVarValue(0,i,j,k))
					{
						if (found)
						{
							std::cout << "error 402: Two values for the"
							     << "same cell!" <<std::endl;
							exit(402);
						}
						else
						{
							found=true;
							std::cout << k+1;
							line.push_back(k+1);
						}
					}
				if (found)
					grid.push_back(line);
				else
				{
					std::cout<< "error 401: missing value at point ("
					    << i << "," << j <<") !" << std::endl;
					exit(401);
				}
			}
		}
	}
	return grid;
}


int main(int argc, char ** argv)
/**
 * @brief Parses the file specified in the first command-line argument
 * and returns the corresponding sudoku grid. Of course, this file
 * must correspond to an assignment as returned by minisat2.
 */
{
	if (argc==1)
	{
		assignment2Grid(&std::cin);
	}
	else
	{
		std::string path (argv[1]);
		std::cout<< "Opening " << path << std::endl;
		std::ifstream infile(path.c_str());
		assignment2Grid(&infile);
		infile.close();
	}
	std::cout<<std::endl<<std::endl;
	return 0;
}
