#include <stdlib.h>
#include <vector>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <sstream>


#include "../cnflib/variableset.h"
#include "../cnflib/clause.h"
#include "../cnflib/formula.h"
#include "kakuro.h"



void assignment2Grid(VariableSet * vars, Kakuro * kak)
{
        std::vector< std::vector<unsigned int> >  gridStruct = kak->getGridStruc();
        std::vector< std::vector<unsigned int> >  horizontalSum = kak->getHorizontalSum();
        std::vector< std::vector<unsigned int> >  verticalSum = kak->getVerticalSum();
        bool found;
        for (unsigned int i=0; i<kak->getNSize(); i++)
        {
                for (unsigned int j=0; j<kak->getMSize(); j++)
                {
                        if ( gridStruct[i][j] != 0) // This cell needs to be filled
                        {
                                found = false;
                                for (unsigned int k=0; k<9; k++)
                                        if (vars->getVarValue(0,i,j,k))
                                        {
                                                if (found)
                                                {
                                                        std::cout << "ERROR: Two values for the"
                                                                  << " same cell!" <<std::endl;
                                                        exit(100);
                                                }
                                                else
                                                {
                                                        found=true;
                                                        kak->assignValue(i,j,k+1);
                                                }
                                        }
                                if (!found)
                                {
                                        std::cout<< "ERROR://  missing value at point ("
                                                 << i << "," << j <<") !" << std::endl;
                                        exit(100);
                                }
                        }
                }
        }
}


int main(int argc, char ** argv)
/**
 * \brief Parses the file specified in the first command-line argument
 * and returns the corresponding sudoku grid. Of course, this file
 * must correspond to an assignment as returned by minisat2.
 */
{

        // parsing the kakuro grid
        Kakuro kak;
        std::string path ("./currentKakuro.txt");
        std::ifstream infile(path.c_str(), std::ifstream::in);
        std::cout<<"Parsing grid"<<std::endl;
        kak.parseGrid(&infile);
        infile.close();
        std::cout<<"Interpreting grid"<<std::endl;
        kak.grids2sums();

        // parsing the assignment as well                
        VariableSet * vars = new VariableSet(kak.getNSize(), kak.getMSize(), 9);
        vars->addVars(kak.getNHints(),12);
        std::cout<<"Parsing dimacs file"<<std::endl;
        if (argc==1)
        {
                vars->parseDimacs(&std::cin);
        }
        else
        {
                std::string path2(argv[1]);
                std::ifstream assignment(argv[1], std::ifstream::in);
//                assignment->open(path2.c_str());
                vars->parseDimacs(&assignment);
                assignment.close();
        }
        assignment2Grid(vars,&kak);
        std::cout<<"Assignment Parsed..."<<std::endl
                 <<"Solution:"<<std::endl<<std::endl;
        kak.printToStream(std::cout,false);
        std::cout<<std::endl<<std::endl;
//        std::cout.flush();
	return 0;
}

