/**
 * \file kakuro.cpp
 * \author Léo Perrin
 * \date 2012/01/24
 */

#include <stdlib.h>
#include <vector>
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <fstream>

#include "kakuro.h"

/**
 * \brief Transforms a C++ string into an unsigned int using stream
 * buffers.
 *
 * \param str The string to turn into a number.
 *
 * \return The corresponding integer.
 */
unsigned int string2uint(std::string str)
{
	std::istringstream buffer(str);
	unsigned int number;
	buffer >> number;
	return number;
}


void Kakuro::parseCell(std::string cell,
                        bool * type,
                        unsigned int * horizontal,
                        unsigned int * vertical )
{
        *type = false;
        if (cell.compare("-")==0)
        {
                *type = true;
                *horizontal = 0;
                *vertical = 0;
        }
        else if (cell.compare("-\\-")==0)
        {
                *horizontal = 0;
                *vertical = 0;
        }
        else
        {
                std::string part1="",part2="";
                int current=1;
                for(unsigned int i=0; i<cell.size(); i++)
                        if (cell[i]=='\\')
                                current=2;
                        else if (current==1)
                                part1 = part1 + cell[i];
                        else if (current==2)
                                part2 = part2 + cell[i];
                if ( (part1.compare("-")!=0) && (part2.compare("-")!=0) )
                {
                        *vertical = string2uint(part1);
                        *horizontal = string2uint(part2);
                }
                else if (part1.compare("-")!=0)
                {
                        *vertical = string2uint(part1);
                        *horizontal = 0;
                }
                else
                {
                        *vertical = 0;
                        *horizontal = string2uint(part2);
                }

        }
}


void Kakuro::printGrids()
{
        std::cout<<"Structure:"<<std::endl;
        for (unsigned int i=0; i<gridStruct.size(); i++)
        {
                std::cout<<std::endl;
                for (unsigned int j=0; j<gridStruct[i].size(); j++)
                        if (gridStruct[i][j])
                                std::cout<<"   0";
                        else
                                std::cout<<"   1";
        }
        std::cout<<std::endl<<"Vertical hints:"<<std::endl;
        for (unsigned int i=0; i<verticalSum.size(); i++)
        {
                std::cout<<std::endl;
                for (unsigned int j=0; j<verticalSum[i].size(); j++)
                        std::cout<<std::setw(4)<<verticalSum[i][j];
        }
        std::cout<<std::endl<<"Horizontal hints:"<<std::endl;
        for (unsigned int i=0; i<horizontalSum.size(); i++)
        {
                std::cout<<std::endl;
                for (unsigned int j=0; j<horizontalSum[i].size(); j++)
                        std::cout<<std::setw(4)<<horizontalSum[i][j];
        }
        std::cout<<std::endl<<std::endl;
}
                

void Kakuro::parseGrid(std::istream * input)
{
        std::string inputLine;
        char x;
	(*input)>>nSize;
        (*input)>>x; // to "jump over" the x in the declaration
        (*input)>>mSize;
        std::vector<unsigned int> strLine; // for the structure
        std::vector<unsigned int> horLine;  // for the horizontal sums
        std::vector<unsigned int> vertLine; // for the vertical sums
        unsigned int j=0;          // FOR GREAT JUSTICE (never mind...)
        while ((*input).good())
        {
                (*input)>>x;
                (*input).putback(x);
                (*input)>>inputLine;
                bool type;
                unsigned int hor, vert;
                parseCell(inputLine,&type,&hor,&vert);
                if (type)
                        strLine.push_back((unsigned int)1);
                else
                        strLine.push_back((unsigned int)0);
                horLine.push_back(hor);
                vertLine.push_back(vert);
                j++;
                if (j==mSize)
                {
                        j=0;
                        gridStruct.push_back(strLine);
                        horizontalSum.push_back(horLine);
                        verticalSum.push_back(vertLine);
                        strLine.clear();
                        horLine.clear();
                        vertLine.clear();
                }
        }
}


void Kakuro::printToStream(std::ostream &out, bool isEmpty)
{
        out<<nSize<<" x "<<mSize;
        for (unsigned int i=0; i<nSize; i++)
        {
                out<<std::endl;
                for (unsigned int j=0; j<mSize; j++)
                {
                        if ( gridStruct[i][j]==0 ) // This cell is empty
                        {
                                if ( (horizontalSum[i][j] != 0)
                                     &&(verticalSum[i][j] != 0) )
                                        out<<std::setw(2)<<std::right<<verticalSum[i][j]
                                              <<"\\"<<std::setw(2)<<std::left<<horizontalSum[i][j];
                                else if ( horizontalSum[i][j] != 0)
                                        out<<" -\\"<<std::setw(2)<<std::left<<horizontalSum[i][j];
                                else if ( verticalSum[i][j] != 0)
                                        out<<std::setw(2)<<std::right<<verticalSum[i][j]<<"\\- ";
                                else
                                        out<<" -\\- ";
                        }
                        else // This cell needs to be filled
                        {
                                if (isEmpty)
                                        out<<"  -  ";
                                else
                                        out<<"  "<<gridStruct[i][j]<<"  ";
                        }
                        out<<" ";
                }
        }
        out<<std::endl<<std::endl;
}


void Kakuro::grids2sums()
{
        // studying horizontal sums
        for (unsigned int i=0; i<nSize; i++)
                for (unsigned int j=0; j<mSize; j++)
                        if (horizontalSum[i][j] != 0)
                        {
                                std::vector<unsigned int> x;
                                std::vector<unsigned int> y;
//                              std::cout<<std::endl;
                                for (unsigned int k=j+1; k<mSize; k++)
                                        if (!gridStruct[i][k]) // we reached a fixed cell
                                                break;
                                        else
                                        {
                                                x.push_back(i);
                                                y.push_back(k);
//                                                std::cout<<"("<<i<<","<<k<<") ";
                                        }
//                              std::cout<<horizontalSum[i][j];
                                sumsValue.push_back(horizontalSum[i][j]);
                                sumsAbscissa.push_back(x);
                                sumsOrdinate.push_back(y);
                        }       
        // studying vertical sums
        for (unsigned int i=0; i<nSize; i++)
                for (unsigned int j=0; j<mSize; j++)
                        if (verticalSum[i][j] != 0)
                        {
                                std::vector<unsigned int> x;
                                std::vector<unsigned int> y;
//                                std::cout<<std::endl;
                                for (unsigned int k=i+1; k<nSize; k++)
                                        if (!gridStruct[k][j]) // we reached a fixed cell
                                                break;
                                        else
                                        {
                                                x.push_back(k);
                                                y.push_back(j);
//                                              std::cout<<"("<<k<<","<<j<<") ";
                                        }
//                                std::cout<<verticalSum[i][j];
                                sumsValue.push_back(verticalSum[i][j]);
                                sumsAbscissa.push_back(x);
                                sumsOrdinate.push_back(y);
                        }
}


void Kakuro::assignValue(unsigned int i, unsigned int j, unsigned int k)
{
        if (i>nSize)
        {
                std::cout<<"ERROR: assigning value out of grid ("
                         <<i<<">"<<nSize<<")"<<std::endl;
                exit(100);
        }
        else if(j>mSize)
        {
                std::cout<<"ERROR: assigning value out of grid ("
                         <<j<<">"<<mSize<<")"<<std::endl;
                exit(100);
        }
        else
                gridStruct[i][j] = k;
}


// ---------------- You want geters? Lots of'em below -----------------

unsigned int Kakuro::getNSize()
{ return nSize; }

unsigned int Kakuro::getMSize()
{ return mSize; }

unsigned int Kakuro::getNHints()
{ return sumsValue.size(); }

std::vector< std::vector<unsigned int> > Kakuro::getGridStruc()
{ return gridStruct;}

std::vector< std::vector<unsigned int> > Kakuro::getHorizontalSum()
{ return horizontalSum; }

std::vector< std::vector<unsigned int> > Kakuro::getVerticalSum()
{ return verticalSum; }

std::vector<unsigned int> Kakuro::getSumsValue()
{ return sumsValue; }

std::vector< std::vector<unsigned int> > Kakuro::getSumsOrdinate()
{ return sumsOrdinate; }

std::vector< std::vector<unsigned int> > Kakuro::getSumsAbscissa()
{ return sumsAbscissa; }


/*
Kakuro::~Kakuro()
{
        std::cout<<"Destruction of a kakuro instance";
}
*/
