/**
 * @file variableset.cpp
 * @author Léo Perrin
 * @date Time-stamp: <2012-05-09 20:58:51 leo>
 */

#include <stdlib.h>
#include <vector>
#include <iostream>
#include <string>
#include <fstream>

#include "variableset.h"

using namespace cnf;


VariableSet::VariableSet(unsigned int i, unsigned int j)
{
        addVars(i,j);
}


VariableSet::VariableSet(unsigned int i, unsigned int j, unsigned int k)
{
        addVars(i,j,k);
}


void VariableSet::addVars(unsigned int i, unsigned int j)
{
        std::vector<unsigned int> itsDimensions;
	itsDimensions.push_back(i);
	itsDimensions.push_back(j);
        dim.push_back(itsDimensions);
        sizes.push_back(i*j);
        std::vector<bool> newVarValue(i*j,false);
        varValue.insert(varValue.end(), newVarValue.begin(),
                        newVarValue.end());
}


void VariableSet::addVars(unsigned int i, unsigned int j, unsigned int k)
{
        std::vector<unsigned int> itsDimensions;
	itsDimensions.push_back(i);
	itsDimensions.push_back(j);
        itsDimensions.push_back(k);
        dim.push_back(itsDimensions);
        sizes.push_back(i*j*k);
        std::vector<bool> newVarValue(i*j*k,false);
        varValue.insert(varValue.end(), newVarValue.begin(),
                        newVarValue.end());
}


unsigned int VariableSet::getVarCode(unsigned int s, unsigned int i, unsigned int j)
{
	if (s > sizes.size())
	{
		std::cout<<"error in VariableSet: wrong subset index"<<" ("
                         <<s<<">="<<sizes.size()<<")"<< std::endl;
		exit(100);
	}
	else if( (i>=dim[s][0]) ||
                 (j>=dim[s][1]) )
	{
		std::cout<<"error in VariableSet: indices out of bound!"<<" ("
                         <<i<<">="<<dim[s][0]<<" or"
                         <<j<<">="<<dim[s][1]<<")"
                         << std::endl;
		exit(100);
	}
	else
	{
		unsigned int varCode=0;
                for (unsigned int subset=0; subset<s; subset++)
                        varCode += sizes[subset];
		varCode += i + (dim[s][0])*j;
		return varCode +1;
	}
}

unsigned int VariableSet::getVarCode(unsigned int s, unsigned int i, unsigned int j, unsigned int k)
{
	if (s > sizes.size())
	{
		std::cout<<"error in VariableSet: wrong subset index"<<" ("
                         <<s<<">="<<sizes.size()<<")"<< std::endl;
		exit(100);
	}
	else if ( (i>=dim[s][0]) || 
             (j>=dim[s][1]) ||
             (k>=dim[s][2]) )
	{
		std::cout<<"error in VariableSet: indices out of bound!"<<" ("
                         <<i<<">="<<dim[s][0]<<" or"
                         <<j<<">="<<dim[s][1]<<" or"
                         <<k<<">="<<dim[s][2]<<")"<< std::endl;
		exit(100);
	}
	else
	{
		unsigned int varCode=0;
                for (unsigned int subset=0; subset<s; subset++)
                        varCode += sizes[subset];
		varCode += i + (dim[s][0])*(j + dim[s][1]*k);

//                std::cout<<"("<<i<<","<<j<<","<<k<<" => "<<varCode+1<<")@t";
		return varCode+1;
	}
}


bool VariableSet::getVarValue(unsigned int s, unsigned int i, unsigned int j)
{
	return varValue[getVarCode(s,i,j) -1];
}


bool VariableSet::getVarValue(unsigned int s, unsigned int i, unsigned int j, unsigned int k)
{
	return varValue[getVarCode(s,i,j,k) -1];
}


void VariableSet::parseDimacs(std::istream * input)
{
	if (!(*input))
	{
		std::cout<<"error 100: No such file!"<<std::endl;
		exit(100);
	}
	else
	{
		std::string res;
		(*input) >> res;
		if (res.compare("SAT") != 0)
		{
			std::cout<<"error 101: The file does not correspond "
				 <<"to a satisfiable formula."<<std::endl;
			exit(101);
		}
		else
		{
			int i=1;
			while ( ((*input) >> i) && (i!=0) )
				if (i>0)
					varValue[i-1] = true;
				else
					varValue[-i+1] = false;
		}
	}
}


unsigned int VariableSet::getCard()
{
        unsigned int card=0;
        for (unsigned int s=0; s<sizes.size(); s++)
                card += sizes[s];
	return card;
}



/*
std::vector<unsigned int> VariableSet::getVarCoord(unsigned int code)
{
	std::vector<unsigned int> coord;
	code -= 1;
	coord.push_back(code % dim[0]);
	code = (code - (code % dim[0]))/dim[0];
	coord.push_back(code % dim[1]);
	if (dim.size()>2)
	{
		code = (code - (code % dim[1]))/dim[1];
		coord.push_back(code % dim[2]);
	}
	return coord;
}
*/



// Thank you "1 to nvars" encoding, thank you. You make it SO easy.


// I hate you.
