/**
 * @file   assignment2slitherlink.cpp
 * @author Léo Perrin
 * @date   Time-stamp: <2012-06-19 16:11:24 leo>
 * 
 * @brief Contains functions used to to read a solution for a
 * slitherink from an assignment returned by minisat2.
 */

#include "../libcnf/libcnf.h"
#include "slitherlink.h"


#define HORIZONTAL true
#define VERTICAL false


int main(void)
{
        // reading the current slitherlink instance
        std::string path ("current_slitherlink.txt");
        std::ifstream infile(path.c_str());
        Slitherlink * sl = new Slitherlink();
        sl->parseInput(&infile);
        unsigned int maxLoopSize = sl->getHeight()*sl->getWidth()*4;

        cnf::VariableSet vars(sl->getHeight()+1,sl->getWidth());
        // vertical edges continuity
        vars.addVars(sl->getHeight(),sl->getWidth()+1);
        // horizontal edges indices
        vars.addVars(sl->getHeight()+1,sl->getWidth(),maxLoopSize+1);
        // vertical edges continuity
        vars.addVars(sl->getHeight(),sl->getWidth()+1,maxLoopSize+1);
        // horizontal edges sense
        vars.addVars(sl->getHeight()+1,sl->getWidth());
        // vertical edges sense
        vars.addVars(sl->getHeight(),sl->getWidth()+1);

        // reading the result of minisat
        vars.parseDimacs(&std::cin);
        
        // assigning the horizontal edges accordingly
        for (unsigned int i=0; i<=sl->getHeight(); i++)
                for (unsigned int j=0; j<sl->getWidth(); j++)
                        sl->setVertex(HORIZONTAL,i,j,
                                      vars.getVarValue(0,i,j));
        
        // assigning the vertical edges accordingly
        for (unsigned int i=0; i<sl->getHeight(); i++)
                for (unsigned int j=0; j<=sl->getWidth(); j++)
                        sl->setVertex(VERTICAL,i,j,
                                      vars.getVarValue(1,i,j));

        // outputing the result
        sl->printSolution();
        return 0;
}

