/**
 * @file   slitherlink.cpp
 * @author Léo Perrin
 * @date   Time-stamp: <2012-05-23 15:39:09 leo>
 * 
 * @brief Contains the actual code of the Slitherlink class.
 */
#include <stdlib.h>
#include <string>

#include "slitherlink.h"


Slitherlink::Slitherlink()
{
        width = 0;
        height = 0;
}


void Slitherlink::parseInput(std::istream *input)
{
        std::string line; // one line of input
        char c;
        (*input) >> height;
        (*input) >> c; // dealing with the "x"
        (*input) >> width;
        for (unsigned int i=0; i<height; i++)
        {
                std::vector<bool> horizontal(width,false);
                std::vector<bool> vertical(width+1,false);
                std::vector<int> hintLine(width);
                for (unsigned int j=0; j<width; j++)
                {
                        (*input) >> c;
                        if (c == '-')
                                hintLine[j] = -1;
                        else
                                hintLine[j] = c - '0';
                }
                hints.push_back(hintLine);
                hVertices.push_back(horizontal);
                vVertices.push_back(vertical);
        }
        // adding the last line to the horizontal vertices array:
        std::vector<bool> lastLine(width,false);
        hVertices.push_back(lastLine);
}


void Slitherlink::setVertex(bool horizontal, unsigned int i,
                            unsigned int j, bool value)
{
        if (
                (horizontal && (j >= width)) ||
                (!horizontal && (i >= height))
                )
        {
                std::cout<<"ERROR in Slitherlink.setVertex() index out"
                        " of range (i="<<i<<",j="<<j<<") while (width="
                         <<width<<", height="<<height<<")"<<std::endl;
                exit(1);
        }
        else if (horizontal)
                hVertices[i][j] = value;
        else
                vVertices[i][j] = value;
}



int Slitherlink:: getHint(unsigned int i, unsigned int j)
{
        if ((i >= height)||(j >= width))
        {
                std::cout<<"ERROR: in Slitherlink.setNode() index out "
                        "of range: (i="<<i<<",j="<<j<<") while (width="
                         <<width<<", height="<<height<<")"<<std::endl;
                exit(1);
        }
        else
                return hints[i][j];
}


unsigned int Slitherlink::getWidth()
{
        return width;
}


unsigned int Slitherlink::getHeight()
{
        return height;
}


// =================================================== printing stuff


void Slitherlink::printNonHintLine(unsigned int i)
{
        for (unsigned int j=0; j<width; j++)
        {
                if (i>0 && i<height)
                {
                        if (vVertices[i-1][j] && vVertices[i][j])
                                std::cout<<"|";
                        else if (j>0)
                        {
                                if (hVertices[i][j-1] && hVertices[i][j])
                                        std::cout<<"_";
                                else
                                        std::cout<<" ";
                        }
                        else
                                std::cout<<" ";
                }
                else
                {
                        if (j>0)
                        {
                                if (hVertices[i][j-1] && hVertices[i][j])
                                        std::cout<<"_";
                                else
                                        std::cout<<" ";
                        }
                        else
                                std::cout<<" ";
                }
                if (hVertices[i][j])
                        std::cout<<"_";
                else
                        std::cout<<" ";
        }
        // printing last column
        if (i>0 && i<height)
                if (vVertices[i-1][width] && vVertices[i][width])
                        std::cout<<"|";
        std::cout<<std::endl;
}


void Slitherlink::printSolution()
{
        for (unsigned int i=0; i<height; i++)
        {
                // printing the vertices on the empty line
                printNonHintLine(i);
                // printing the hints and the vertical vertices
                for (unsigned int j=0; j<width; j++)
                {
                        if (vVertices[i][j])
                                std::cout<<"|";
                        else
                                std::cout<<" ";
                        if (hints[i][j] != -1)
                                std::cout<<hints[i][j];
                        else
                                std::cout<<" ";
                }
                // printing the last vertical vertex if any
                if (vVertices[i][width])
                        std::cout<<"|"<<std::endl;
                else
                        std::cout<<" "<<std::endl;
        }
        printNonHintLine(height);
}
